//****************************************************
// fish tank / coffee pot thermostat
//  ** last updated 11-21-12
//
// copyright (c) 2012 by Derek Tombrello
//
// P1.1 = thermistor
// P1.5 = heater relay
// P1.6 = tens digit selector
// P1.7 = ones digit selector
// P2.0,1,2,3 = BCD decoder ABCD
//
//****************************************************

#include <msp430g2452.h>
#include "thermistor.h"

#ifndef TIMER0_A1_VECTOR
#define TIMER0_A1_VECTOR    TIMERA1_VECTOR
#define TIMER0_A0_VECTOR    TIMERA0_VECTOR
#endif

void ConfigClocks(void);
void ConfigPins(void);
void delay_ms(unsigned int ms);
void delay_us(unsigned int us);
void FaultRoutine(void);

int tens = 0, ones = 0, temp = 0;

void main(void)
{

	ConfigClocks();
	ConfigPins();
	initThermistor();

	while (1)
	{
		P2OUT = tens;					// display tens digit of temperature
		P1OUT |= BIT6;
		delay_ms(10);
		P1OUT &= ~BIT6;

		P2OUT = ones;					// display ones digit of temperature
		P1OUT |= BIT7;
		delay_ms(10);
		P1OUT &= ~BIT7;
	}
	
}

void ConfigClocks(void)
{
	WDTCTL = WDTPW + WDTHOLD;          		// Stop watchdog timer

	if (CALBC1_1MHZ ==0xFF || CALDCO_1MHZ == 0xFF)
		FaultRoutine();	         			// If calibration data is erased
 				               	   	   		// run FaultRoutine()
	BCSCTL1 = CALBC1_1MHZ; 		   			// Set range
	DCOCTL = CALDCO_1MHZ;  		   			// Set DCO step + modulation
	BCSCTL3 |= LFXT1S_2;                	// LFXT1 = VLO
	IFG1 &= ~OFIFG;                     	// Clear OSCFault flag
	BCSCTL2 = 0;  							// MCLK & SMCLK = DCO/1

	CCTL0 = CCIE;							// enable TIMER0 interrupt
	CCR0 = 15000;							// set to generate interrupt every 10 sec
	TACTL = TASSEL_1 + ID_3 + MC_1;			// configure TIMER0 - ACLK/8 + Up.To.CCR0

//	_bis_SR_register(LPM3_bits + GIE); 	// Enter LPM3 with interrupts
	_bis_SR_register(GIE);

}

void ConfigPins(void)
{
	P1SEL = P2SEL = 0; 						// set PORT1 & 2 GPIO
	P2DIR = 0xFF;							// set PORT2 as output
	P1DIR = ~(BIT1);						// set P1.1 as input, all others output
	P1OUT = P2OUT = 0;						// clear all PORT1 and PORT2 outputs
}

/* These two functions stop the main thread for a certain number of milli -or- microseconds.
   They are based on trial and error, but they work fine for the out-of-the-box Launchpad board.

   based on default 1 MHz clock speed
*/
void delay_ms(unsigned int ms )
{
      unsigned int i;
      for (i = 0; i<= ms; i++)
        __delay_cycles(500); //Built-in function that suspends the execution for 500 cycles
}

void delay_us(unsigned int us )
{
      unsigned int i;
      for (i = 0; i<= us/2; i++)
        __delay_cycles(1);
}

// TIMER0 interrupt service routine
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_ISR (void)
{
	temp = readTemp();
	tens = temp / 10;
	ones = temp % 10;

	if (temp <= 74 )						// if <= 74 degrees, turn heater on
		P1OUT |= BIT5;


	if (temp >= 78)							// if >= 78 degrees, turn heater off
		P1OUT &= ~(BIT5);
}

void FaultRoutine(void)
 {
   while(1) 			            		// TRAP
   {
		P2OUT = 9;							// display 9 in tens digit of temperature
		P1OUT |= BIT6;
		delay_ms(10);
		P1OUT &= ~BIT6;

		P2OUT = 9;							// display 9 in ones digit of temperature
		P1OUT |= BIT7;
		delay_ms(10);
		P1OUT &= ~BIT7;

   }
 }
