/*
* Inspired by http://reprap.org/wiki/Temperature_Sensor_1_0
* No license info from that page, but some may claim it's GPL.
* Not sure if that is relevant for such a short code snippet.
*/

#include <msp430g2452.h>
#include "thermistor.h"

// Thermistor NT03 50169 - http://www.ametherm.com/datasheetspdf/NT03%2050169.pdf 

// r0: 500
// t0: 25
// r1: 0
// r2: 1600
// beta: 3500
// max adc: 1023
#define NUMTEMPS 40
const short temptable[NUMTEMPS][2] =
{
   {1, 312},
   {27, 103},
   {53, 77},
   {79, 62},
   {105, 52},
   {131, 45},
   {157, 39},
   {183, 34},
   {209, 30},
   {235, 26},
   {261, 22},
   {287, 19},
   {313, 16},
   {339, 13},
   {365, 11},
   {391, 8},
   {417, 6},
   {443, 3},
   {469, 1},
   {495, 0},
   {521, -2},
   {547, -4},
   {573, -6},
   {599, -8},
   {625, -10},
   {651, -13},
   {677, -15},
   {703, -17},
   {729, -19},
   {755, -21},
   {781, -24},
   {807, -26},
   {833, -29},
   {859, -32},
   {885, -35},
   {911, -39},
   {937, -44},
   {963, -49},
   {989, -57},
   {1015, -74}
};

void initThermistor()
{
	WDTCTL = WDTPW + WDTHOLD;               	// Stop WDT
	ADC10CTL1 = INCH_1;             			// Conversion code signed format, input A1
	ADC10AE0 |= BIT1;                         	// P1.1 ADC option select
	ADC10CTL0 = ADC10SHT_2 + ADC10ON; 			// ADC10ON, ( include '+ ADC10IE' for interrupt enabled)
}

int rawValueThermistor()
{
	ADC10CTL0 |= ENC + ADC10SC;					// Sampling and conversion start
//    __bis_SR_register(CPUOFF + GIE);			// LPM0, ADC10_ISR will force exit

    return ADC10MEM;
}

int readTemp()
{
	int rawtemp = rawValueThermistor();
	int current_celsius = 0;
	short current_farenheit = 0;

	int i = 0;
	for (i=1;i < NUMTEMPS;i++)
	{
		if (temptable[i][0] > rawtemp)
		{
			int realtemp  = temptable[i-1][1] + (rawtemp - temptable[i-1][0]) * (temptable[i][1] - temptable[i-1][1]) / (temptable[i][0] - temptable[i-1][0]);

			if (realtemp > 255)
				realtemp = 255;

			current_celsius = realtemp;

			break;
		}
	}

// Overflow: We just clamp to 0 degrees celsius
	if (i == NUMTEMPS)
	{
		current_celsius = 0;
	}

	current_farenheit = (current_celsius * 1.8) + 32;

	return current_farenheit;
}

#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
//	__bic_SR_register_on_exit(CPUOFF);			// Clear CPUOFF bit from 0(SR)
}
